/*
 * Dynamic pointers array.
 *
 * This software is distributed under the terms of GNU
 * Public License version 2 or later versions.
 *
 */

#ifndef _PARRAY_H
#define _PARRAY_H

/* Callback type used to compare elements in array via pointers. */
typedef int (*parray_compare_t)(const void *a, const void *b);

/* Callback used for traversion of array elements. */
typedef int (*parray_callback_t)(void *ptr, void *opaque);

struct parray {
	/* Number of allocated pointers in data. */
	size_t		nelems;

	/* Number of occupied pointers in data. */
	size_t		nalloc;

	/* Array of pointers of size data[nalloc]. */
	void		**data;
}

/* Create new pointers array. */
struct parray *parray_new(size_t init_size);

/* Free pointers array and it's resources. */
void parray_free(struct parray **array);

/* Add pointer to array. */
unsigned int parray_add(struct parray *array, void *ptr);

/* Remove pointer from array. */
void parray_delete(struct parray *array, void *ptr);

/* Delete pointer from array at position nth. */
void parray_delete_nth(struct parray *array, unsigned int nth);

/* Delete range of pointers from array starting from nth up to count element. */
void parray_delete_range(struct parray *array, unsigned int nth, size_t count);

/* Change pointer at position nth. */
void parray_change(struct parray *array, void *ptr, unsigned int nth);

/* Get pointer at position pos. */
void *parray_get_nth(struct parray *array, unsigned int nth);

/* Sort array of pointers using compare function compare_func. */
void parray_sort(struct parray *array, parray_compare_t compare_func);

/* Search sorted array of pointers using compare function compare_func. */
unsigned int parray_search(struct parray *array, void *ptr, parray_compare_t compare_func);

/* Traverse pointer array calling callback function for each element. */
void *parray_foreach(struct parray *array, parray_callback_t callback, void *opaque);

#endif
